@@ -2,7 +2,7 @@
 #
 # ChartColumn - A class for writing the Excel XLSX Column charts.
 #
-# Copyright 2013-2016, John McNamara, jmcnamara@cpan.org
+# Copyright 2013-2018, John McNamara, jmcnamara@cpan.org
 #
 
 from . import chart
@@ -95,9 +95,6 @@ class ChartColumn(chart.Chart):
         for data in series:
             self._write_ser(data)
 
-        # Write the c:marker element.
-        self._write_marker_value()
-
         # Write the c:gapWidth element.
         if args['primary_axes']:
             self._write_gap_width(self.series_gap_1)
