@@ -1144,31 +1144,43 @@ class Workbook(xmlwriter.XMLwriter):
         image_name = os.path.basename(filename)
 
         # Look for some common image file markers.
-        marker1 = (unpack('3s', data[1:4]))[0]
-        marker2 = (unpack('>H', data[:2]))[0]
-        marker3 = (unpack('2s', data[:2]))[0]
+        marker1 = unpack('3s', data[1:4])[0]
+        marker2 = unpack('>H', data[:2])[0]
+        marker3 = unpack('2s', data[:2])[0]
+        marker4 = unpack('<L', data[:4])[0]
+        marker5 = (unpack('4s', data[40:44]))[0]
 
         if sys.version_info < (2, 6, 0):
             # Python 2.5/Jython.
             png_marker = 'PNG'
             bmp_marker = 'BM'
+            emf_marker = ' EMF'
         else:
             # Eval the binary literals for Python 2.5/Jython compatibility.
             png_marker = eval("b'PNG'")
             bmp_marker = eval("b'BM'")
+            emf_marker = eval("b' EMF'")
 
         if marker1 == png_marker:
-            self.image_types['png'] = 1
+            self.image_types['png'] = True
             (image_type, width, height, x_dpi, y_dpi) = self._process_png(data)
 
         elif marker2 == 0xFFD8:
-            self.image_types['jpeg'] = 1
+            self.image_types['jpeg'] = True
             (image_type, width, height, x_dpi, y_dpi) = self._process_jpg(data)
 
         elif marker3 == bmp_marker:
-            self.image_types['bmp'] = 1
+            self.image_types['bmp'] = True
             (image_type, width, height) = self._process_bmp(data)
 
+        elif marker4 == 0x9AC6CDD7:
+            self.image_types['wmf'] = True
+            (image_type, width, height, x_dpi, y_dpi) = self._process_wmf(data)
+
+        elif marker4 == 1 and marker5 == emf_marker:
+            self.image_types['emf'] = True
+            (image_type, width, height, x_dpi, y_dpi) = self._process_emf(data)
+
         else:
             raise Exception("%s: Unknown or unsupported image file format."
                             % filename)
@@ -1210,19 +1222,19 @@ class Workbook(xmlwriter.XMLwriter):
         # IHDR element. Also read the DPI in the pHYs element.
         while not end_marker and offset < data_length:
 
-            length = (unpack('>I', data[offset + 0:offset + 4]))[0]
-            marker = (unpack('>I', data[offset + 4:offset + 8]))[0]
+            length = unpack('>I', data[offset + 0:offset + 4])[0]
+            marker = unpack('>I', data[offset + 4:offset + 8])[0]
 
             # Read the image dimensions.
             if marker == marker_ihdr:
-                width = (unpack('>I', data[offset + 8:offset + 12]))[0]
-                height = (unpack('>I', data[offset + 12:offset + 16]))[0]
+                width = unpack('>I', data[offset + 8:offset + 12])[0]
+                height = unpack('>I', data[offset + 12:offset + 16])[0]
 
             # Read the image DPI.
             if marker == marker_phys:
-                x_density = (unpack('>I', data[offset + 8:offset + 12]))[0]
-                y_density = (unpack('>I', data[offset + 12:offset + 16]))[0]
-                units = (unpack('b', data[offset + 16:offset + 17]))[0]
+                x_density = unpack('>I', data[offset + 8:offset + 12])[0]
+                y_density = unpack('>I', data[offset + 12:offset + 16])[0]
+                units = unpack('b', data[offset + 16:offset + 17])[0]
 
                 if units == 1:
                     x_dpi = x_density * 0.0254
@@ -1249,8 +1261,8 @@ class Workbook(xmlwriter.XMLwriter):
         # Search through the image data to read the JPEG markers.
         while not end_marker and offset < data_length:
 
-            marker = (unpack('>H', data[offset + 0:offset + 2]))[0]
-            length = (unpack('>H', data[offset + 2:offset + 4]))[0]
+            marker = unpack('>H', data[offset + 0:offset + 2])[0]
+            length = unpack('>H', data[offset + 2:offset + 4])[0]
 
             # Read the height and width in the 0xFFCn elements (except C4, C8
             # and CC which aren't SOF markers).
@@ -1258,14 +1270,14 @@ class Workbook(xmlwriter.XMLwriter):
                     and marker != 0xFFC4
                     and marker != 0xFFC8
                     and marker != 0xFFCC):
-                height = (unpack('>H', data[offset + 5:offset + 7]))[0]
-                width = (unpack('>H', data[offset + 7:offset + 9]))[0]
+                height = unpack('>H', data[offset + 5:offset + 7])[0]
+                width = unpack('>H', data[offset + 7:offset + 9])[0]
 
             # Read the DPI in the 0xFFE0 element.
             if marker == 0xFFE0:
-                units = (unpack('b', data[offset + 11:offset + 12]))[0]
-                x_density = (unpack('>H', data[offset + 12:offset + 14]))[0]
-                y_density = (unpack('>H', data[offset + 14:offset + 16]))[0]
+                units = unpack('b', data[offset + 11:offset + 12])[0]
+                x_density = unpack('>H', data[offset + 12:offset + 14])[0]
+                y_density = unpack('>H', data[offset + 14:offset + 16])[0]
 
                 if units == 1:
                     x_dpi = x_density
@@ -1291,10 +1303,64 @@ class Workbook(xmlwriter.XMLwriter):
 
     def _process_bmp(self, data):
         # Extract width and height information from a BMP file.
-        width = (unpack('<L', data[18:22]))[0]
-        height = (unpack('<L', data[22:26]))[0]
+        width = unpack('<L', data[18:22])[0]
+        height = unpack('<L', data[22:26])[0]
         return 'bmp', width, height
 
+    def _process_wmf(self, data):
+        # Extract width and height information from a WMF file.
+        x_dpi = 96
+        y_dpi = 96
+
+        # Read the bounding box, measured in logical units.
+        x1 = unpack("<h", data[6:8])[0]
+        y1 = unpack("<h", data[8:10])[0]
+        x2 = unpack("<h", data[10:12])[0]
+        y2 = unpack("<h", data[12:14])[0]
+
+        # Read the number of logical units per inch. Used to scale the image.
+        inch = unpack("<H", data[14:16])[0]
+
+        # Convert to rendered height and width.
+        width = float((x2 - x1) * x_dpi) / inch
+        height = float((y2 - y1) * y_dpi) / inch
+
+        return 'wmf', width, height, x_dpi, y_dpi
+
+    def _process_emf(self, data):
+        # Extract width and height information from a EMF file.
+
+        # Read the bounding box, measured in logical units.
+        bound_x1 = unpack("<l", data[8:12])[0]
+        bound_y1 = unpack("<l", data[12:16])[0]
+        bound_x2 = unpack("<l", data[16:20])[0]
+        bound_y2 = unpack("<l", data[20:24])[0]
+
+        # Convert the bounds to width and height.
+        width = bound_x2 - bound_x1
+        height = bound_y2 - bound_y1
+
+        # Read the rectangular frame in units of 0.01mm.
+        frame_x1 = unpack("<l", data[24:28])[0]
+        frame_y1 = unpack("<l", data[28:32])[0]
+        frame_x2 = unpack("<l", data[32:36])[0]
+        frame_y2 = unpack("<l", data[36:40])[0]
+
+        # Convert the frame bounds to mm width and height.
+        width_mm = 0.01 * (frame_x2 - frame_x1)
+        height_mm = 0.01 * (frame_y2 - frame_y1)
+
+        # Get the dpi based on the logical size.
+        x_dpi = width * 25.4 / width_mm
+        y_dpi = height * 25.4 / height_mm
+
+        # This is to match Excel's calculation. It is probably to account for
+        # the fact that the bounding box is inclusive-inclusive. Or a bug.
+        width += 1
+        height += 1
+
+        return 'emf', width, height, x_dpi, y_dpi
+
     def _extract_named_ranges(self, defined_names):
         # Extract the named ranges from the sorted list of defined names.
         # These are used in the App.xml file.
